/*
 * Copyright (C) 2002-2011 OSS Nokalva.  All rights reserved.
 */

/*
 * THIS FILE IS PROPRIETARY MATERIAL OF OSS NOKALVA, INC. AND MAY BE USED
 * ONLY BY DIRECT LICENSEES OF OSS NOKALVA, INC. THIS FILE MAY NOT BE
 * DISTRIBUTED.
 */

/*
 * This example consists of 2 console executables, "server" and "client".
 * First run the server and then the client from 2 separate console windows.
 * The client reads a file with BER encoded concatenated PDUs based on the
 * bcas.asn syntax, then writes the content to a TCP/IP socket. The server--
 * listening to the socket by that time--then reads all the PDUs from the
 * socket and decodes them. The VC++ example requires the OSS socket memory
 * manager, osssmgmt.obj, to be linked with, along with the /MT static
 * library soeddefa.lib. You should use the /MT C-compiler option when
 * C-compiling. The Solaris project requires osssmgmt.o to be linked with.
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
typedef unsigned int SOCKET;
#include <sys/types.h>
#include <netinet/in.h>
#include <sys/socket.h>
#define BSD_COMP
#include <sys/ioctl.h>
#include "bcas.h"  /* compiler-generated header file */
#define INVALID_SOCKET -1


int main(void)
{

    OssGlobal		w, *world = &w;
    char		portId[16 + 1];
    char		ipAddr[16 + 1];
    unsigned char	*buffer;	/* address of the encoder output buffer */
    char		encFileName[32];  /* encoded data file name */
    FILE		*encFile;	
    size_t		fsize;
    int			rc, bytesLeft, bytesSent;
    unsigned char	*buf;
    struct sockaddr_in skadr, *skadrp = &skadr;
    SOCKET		clientSock = (unsigned int)INVALID_SOCKET;


	/*
	 * Call ossinit() first to initialize OSS's global structure
	 */
    if (rc = ossinit(world, bcas)) {
	ossPrint(world, "Ossinit() returned %d\n", rc);
	return rc;
    }

    ossSetEncodingFlags(world, DEBUGPDU|DEFINITE);
    ossSetDecodingFlags(world, DEBUGPDU);

	/*
	 * If you want to decode encoded data from a TCP/IP socket,
	 * then do the following:
	 */
    if (clientSock == INVALID_SOCKET) {
		/*
		 * Create socket
		 */
	clientSock = socket(AF_INET, SOCK_STREAM, 0);
	if (clientSock == INVALID_SOCKET) {
	    ossPrint(world, "socket() failed with error code = %d\n",
							INVALID_SOCKET);
	    return clientSock;
	}
		/*
		 * Bind socket to portId
		 */
	strcpy(portId, "2123");
	strcpy(ipAddr,"127.0.0.1");		/* use local host */
		/*
		 * Set up connect address
		 */
	memset(&skadr, 0, sizeof(skadr));
	skadrp->sin_family = AF_INET;
	skadrp->sin_port = htons((unsigned short)atoi(portId));
	skadrp->sin_addr.s_addr = inet_addr(ipAddr);
	if (skadrp->sin_addr.s_addr == INADDR_NONE) {
	    ossPrint(world, "inet_addr() failed with error code = %d\n",
							INADDR_NONE);
	    return skadrp->sin_addr.s_addr;
	}
		/*
		 * Try to connect
		 */
	rc = connect(clientSock, (struct sockaddr *)skadrp, sizeof(skadr));
	if (rc) {
	    ossPrint(world, "connect() failed with error code = %d\n",
							rc);
	    return rc;
	}
    }
	/*
	 * Set the name of the input file with encoded data
	 */
    if (OSS_BER == ossGetEncodingRules(world))
	strcpy(encFileName, "bcas.ber");
    else if (OSS_PER_ALIGNED == ossGetEncodingRules(world))
	strcpy(encFileName, "bcas.per");
    else
	strcpy(encFileName, "bcas.uper");

    if (!(encFile = fopen(encFileName, "rb"))) {
	ossPrint(world, "\nCannot open an input file\n");
        return 1;
    }
    if (fseek(encFile, 0, SEEK_END) || (fsize = ftell(encFile)) < 0 ||
					fseek(encFile, 0, SEEK_SET)) {
	fclose(encFile);
	ossPrint(world, "\nError reading from an input file\n");
        return 1;
    }

    buffer = malloc(fsize);
    if (fread(buffer, 1, fsize, encFile) != fsize) {
	fclose(encFile);
	ossPrint(world, "Error reading from an input file\n");
	free(buffer);
        return 1;
    }

    ossPrint(world, "Writing the file contents to a socket...\n\n");

    bytesLeft = fsize;
    buf = buffer;

    while(bytesLeft > 0) {
	bytesSent = send(clientSock, (char *)buf, bytesLeft, 0);
	if (bytesSent < 0) {
	    ossPrint(world, "Error writing to a socket\n");
	    free(buffer);
            return 1;
	}
	else {
	    buf += bytesSent;
	    bytesLeft -= bytesSent;
	}
    }
    ossPrint(world, "Data successfully written to a socket, in %ld bytes:\n",
								fsize);
/*    getchar();*/
    close(clientSock);
    free(buffer);
    ossterm(world);
    return 0;
}

