/*
 * Copyright (C) 2002-2011 OSS Nokalva.  All rights reserved.
 */

/*
 * THIS FILE IS PROPRIETARY MATERIAL OF OSS NOKALVA, INC. AND MAY BE USED
 * ONLY BY DIRECT LICENSEES OF OSS NOKALVA, INC. THIS FILE MAY NOT BE
 * DISTRIBUTED.
 */

/*
 * This example consists of 2 console executables, "server" and "client".
 * First run the server and then the client from 2 separate console windows.
 * The client reads a file with BER encoded concatenated PDUs based on the
 * bcas.asn syntax, then writes the content to a TCP/IP socket. The server--
 * listening to the socket by that time--then reads all the PDUs from the
 * socket and decodes them. The VC++ example requires the OSS socket memory
 * manager, osssmgmt.obj, to be linked with, along with the /MT static
 * library soeddefa.lib. You should use the /MT C-compiler option when
 * C-compiling. The Solaris project requires osssmgmt.o to be linked with.
 */

#include <stdio.h>
typedef unsigned int SOCKET;
#include <sys/types.h>
#include <netinet/in.h>
#include <sys/socket.h>
#define BSD_COMP
#include <sys/ioctl.h>
#include <netdb.h>
#include "bcas.h"  /* compiler-generated header file */
#define INVALID_SOCKET -1


int main(void)
{

    OssGlobal	 w, *world = &w;    /* pointer to OSS's global structure */
    OssBuf	 encodedData;       /* OSS encoder/decoder output/input */
    int		 rc = 0, error = 0; /* Return & error codes */
    int		 pdu_num = 1, len;
    BBCard	*myCardPtr = NULL;  /* address of decoded data */
    char	 portId[16 + 1];
    int		 i;
    OssBufExtended startBuf;
    SOCKET	 clientSocket, servSock = (unsigned int)INVALID_SOCKET;
    struct sockaddr_in skadr, *skadrp = &skadr;
    struct sockaddr_in cli_skadr;


	/*
	 * Call ossinit() first to initialize OSS's global structure.
	 */
    if (rc = ossinit(world, bcas)) {
	ossPrint(world, "Ossinit() returned %d\n", rc);
	return rc;
    }

    ossSetEncodingFlags(world, DEBUGPDU);
    ossSetDecodingFlags(world, DEBUGPDU);

	/*
	 * Print the input to the encoder
	 */
    ossPrint(world, "The input to the encoder ...\n\n");
    ossPrintPDU(world, BBCard_PDU, &myCard);
	/*
	 * If you want to decode encoded data from a TCP/IP socket,
	 * then do the following:
	 */
    if (servSock == INVALID_SOCKET) {
		/*
		 * Create socket
		 */
	servSock = socket(AF_INET, SOCK_STREAM, 0);
	if (servSock == INVALID_SOCKET) {
	    ossPrint(world, "socket() failed with error code = %d\n",
							INVALID_SOCKET);
	    return servSock;
	}
		/*
		 * Bind socket to portId
		 */
	strcpy(portId, "2123");
	memset(&skadr, 0, sizeof(skadr));
	skadrp->sin_family = PF_INET;
	skadrp->sin_port = htons((unsigned short)atoi(portId));
	skadrp->sin_addr.s_addr = INADDR_ANY;
	rc = bind(servSock, (struct sockaddr *)skadrp, sizeof(skadr));
	if (rc) {
	    ossPrint(world, "bind() failed with error code = %d\n",
							rc);
	    close(servSock);
	    return rc;
	}
    }
	/*
	 * Listen on socket for Encoded data
	 */
    ossPrint(world, "\n\tWaiting to receive PDU\n");
    if (rc = listen(servSock, 5)) {
	ossPrint(world, "Error trying to listen on socket Id = %d\n", servSock);
	ossPrint(world, "listen() failed with error code = %d\n",
							rc);
	close(servSock);
	return INVALID_SOCKET;
    }
    len = sizeof(cli_skadr);
    clientSocket = accept(servSock, (struct sockaddr *)&cli_skadr, &len);
    if (clientSocket == INVALID_SOCKET) {
	ossPrint(world, "accept() failed with error code = %d\n",
							INVALID_SOCKET);
	close(servSock);
	return INVALID_SOCKET;
    }
		/*
		 * Decode the PDU that was just written to a socket
		 */
    ossPrint(world, "\n\tDecoding from a socket...\n\n");
	/*
	 * Set up the decoder input file name so the decoder decodes
	 * from the socket and the file
	 */
    encodedData.length = 0;	/* keeps length of undecoded bytes left */
    encodedData.value = (unsigned char *)ossMarkObj(world, OSS_SOCKET,
							(void *)clientSocket);

    for (i = 1; !error; i++) {
	/*
	 * To decode from a TCP socket, mark the object with ossMarkObj()
	 */
	ossPrint(world, "\nDecoding PDU #%d from socket...\n\n", i);
	if (rc = ossDecode(world, &pdu_num, &encodedData, (void **)&myCardPtr)) {
	    ossPrint(world, "%s\n", ossGetErrMsg(world));  /* an error occurred, print errmsg */
	    error = rc;
	}
	else {
			/*
			 * Print out the player's batting average
			 */
	    ossPrint(world, "%s of the %s has a batting average of %0.3f\n",
	    myCardPtr->name, myCardPtr->team, myCardPtr->batting_average);
			/*
			 * Print the decoder's output
			 */
	    ossPrint(world, "\nDecoded PDU...\n\n");
	    ossPrintPDU(world, pdu_num, myCardPtr);
	    ossFreePDU(world, pdu_num, myCardPtr);
	    myCardPtr = NULL;
	}
	if (!ossGetBuffer(world, &startBuf)) {
            if (ossSetBuffer(world, &startBuf))
		if (startBuf.value)
		    (*world->freep)(startBuf.value);
	}
    }

    error = rc;
	/*
	 * If encoding to a socket, close socket and terminate the
	 * TCP/IP session
	 */
    close(servSock);
    close(clientSocket);
    ossterm(world);
    return error;
}

